import 'dart:async';

import 'package:amap_track_fluttify/src/android/android.export.g.dart';
import 'package:amap_track_fluttify/src/ios/AMapTrackManager.g.dart';
import 'package:amap_track_fluttify/src/ios/AMapTrackManagerOptions.g.dart';
import 'package:amap_track_fluttify/src/ios/ios.export.g.dart';
import 'package:flutter/cupertino.dart';
import 'package:foundation_fluttify/foundation_fluttify.dart';

import 'models.dart';

part 'delegates.dart';

class AmapTrack extends _Holder with _Add, _Query, _Action, _Pro {
  static final AmapTrack instance = AmapTrack._();

  AmapTrack._();

  Future<void> init({
    @required String iosKey,
    @required int serviceId,
  }) async {
    return platform(
      android: (pool) async {
        _serviceId = serviceId;

        final context = await android_app_Application.get();
        _androidManager = await com_amap_api_track_AMapTrackClient
            .create__android_content_Context(context);
        await _androidManager.setOnTrackListener(_androidDelegate);
      },
      ios: (pool) async {
        _serviceId = serviceId;

        await AmapCore.init(iosKey);
        final option = await AMapTrackManagerOptions.create__();
        await option.set_serviceID(serviceId.toString());

        // AMapTrackManager原生的init方法会返回null, 需要使用initWithOptions进行初始化
        _iosManager = await AMapTrackManager.create__(init: false);
        await _iosManager.initWithOptions(option);
        await _iosManager.set_delegate(_iosDelegate);
        await _iosManager.set_allowsBackgroundLocationUpdates(true);
        await _iosManager.set_pausesLocationUpdatesAutomatically(false);
      },
    );
  }
}

class _Holder {
  AMapTrackManager _iosManager;
  com_amap_api_track_AMapTrackClient _androidManager;

  _AndroidDelegate _androidDelegate = _AndroidDelegate();
  _IOSDelegate _iosDelegate = _IOSDelegate();

  int _serviceId;
}

mixin _Action on _Holder {
  /// 开启寻迹服务
  Future<void> startService(int terminalId, {int trackId}) async {
    assert(terminalId != null);
    return platform(
      android: (pool) async {
        final completer = Completer<void>();
        final option = await com_amap_api_track_TrackParam.create__long__long(
            _serviceId, terminalId);
        if (trackId != null) {
          await option.setTrackId(trackId);
        }
        await _androidManager.startTrack(
          option,
          _androidDelegate
            .._onStartService = completer.complete
            .._onRequestFailed = completer.completeError,
        );

        pool.add(option);
        return completer.future;
      },
      ios: (pool) async {
        final completer = Completer<void>();

        final option = await AMapTrackManagerServiceOption.create__();
        await option.set_terminalID(terminalId.toString());

        if (trackId != null) {
          await _iosManager.set_trackID(trackId.toString());
        }
        await _iosManager.set_delegate(
          _iosDelegate
            .._onStartService = completer.complete
            .._onRequestFailed = completer.completeError,
        );

        await _iosManager.startServiceWithOptions(option);

        pool.add(option);
        return completer.future;
      },
    );
  }

  /// 开始收集上报轨迹数据
  Future<void> startGather(int terminalId) async {
    assert(terminalId != null);
    return platform(
      android: (pool) async {
        final completer = Completer();
        await _androidManager.startGather(
          _androidDelegate
            .._onStartGather = completer.complete
            .._onRequestFailed = completer.completeError,
        );
        return completer.future;
      },
      ios: (pool) async {
        final completer = Completer();
        await _iosManager.set_delegate(
          _iosDelegate
            .._onStartGather = completer.complete
            .._onRequestFailed = completer.completeError,
        );
        await _iosManager.startGatherAndPack();
        return completer.future;
      },
    );
  }

  /// 停止收集上报轨迹数据
  Future<void> stopGather() async {
    return platform(
      android: (pool) async {
        final completer = Completer();
        await _androidManager.stopGather(
          _androidDelegate
            .._onStopGather = completer.complete
            .._onRequestFailed = completer.completeError,
        );
        return completer.future;
      },
      ios: (pool) async {
        final completer = Completer();
        await _iosManager.set_delegate(
          _iosDelegate
            .._onStopGather = completer.complete
            .._onRequestFailed = completer.completeError,
        );
        await _iosManager.stopGaterAndPack();
        return completer.future;
      },
    );
  }

  /// 停止寻迹服务
  Future<void> stopService(int terminalId) async {
    assert(terminalId != null);
    return platform(
      android: (pool) async {
        final completer = Completer();
        final option = await com_amap_api_track_TrackParam.create__long__long(
            _serviceId, terminalId);
        await _androidManager.stopTrack(
          option,
          _androidDelegate
            .._onStopService = completer.complete
            .._onRequestFailed = completer.completeError,
        );
        pool.add(option);
        return completer.future;
      },
      ios: (pool) async {
        final completer = Completer();
        await _iosManager.set_delegate(
          _iosDelegate
            .._onStopService = completer.complete
            .._onRequestFailed = completer.completeError,
        );
        await _iosManager.stopService();
        return completer.future;
      },
    );
  }
}

mixin _Add on _Holder {
  /// 根据终端名称创建终端
  Future<int> addTerminal(String terminalName) async {
    assert(terminalName != null);
    return platform(
      android: (pool) async {
        final completer = Completer<int>();
        final request = await com_amap_api_track_query_model_AddTerminalRequest
            .create__String__long(terminalName, _serviceId);
        await _androidManager.addTerminal(
          request,
          _androidDelegate
            .._onAddTerminal = completer.complete
            .._onRequestFailed = completer.completeError,
        );
        pool.add(request);
        return completer.future;
      },
      ios: (pool) async {
        final completer = Completer<int>();

        final request = await AMapTrackAddTerminalRequest.create__();
        await request.set_serviceID(_serviceId.toString());
        await request.set_terminalName(terminalName);

        await _iosManager.set_delegate(
          _iosDelegate
            .._onAddTerminal = completer.complete
            .._onRequestFailed = completer.completeError,
        );

        await _iosManager.AMapTrackAddTerminal(request);

        pool.add(request);
        return completer.future;
      },
    );
  }
}

mixin _Query on _Holder {
  /// 根据终端名称查询终端信息
  Future<QueryTerminalResponse> queryTerminal(String terminalName) async {
    assert(terminalName != null);
    return platform(
      android: (pool) async {
        final completer = Completer<QueryTerminalResponse>();
        final request =
            await com_amap_api_track_query_model_QueryTerminalRequest
                .create__long__String(_serviceId, terminalName);
        await _androidManager.queryTerminal(
          request,
          _androidDelegate.._onQueryTerminalResponse = completer.complete,
        );
        pool.add(request);
        return completer.future;
      },
      ios: (pool) async {
        final completer = Completer<QueryTerminalResponse>();

        final request = await AMapTrackQueryTerminalRequest.create__();
        await request.set_serviceID(_serviceId.toString());
        await request.set_terminalName(terminalName);

        await _iosManager.set_delegate(
          _iosDelegate
            .._onQueryTerminalResponse = completer.complete
            .._onRequestFailed = completer.completeError,
        );

        await _iosManager.AMapTrackQueryTerminal(request);

        pool.add(request);
        return completer.future;
      },
    );
  }

  /// 查询最后一次上报的位置
  ///
  /// 可以使用服务id和终端id查询某个终端最后一次上报的位置信息
  Future<TrackPoint> queryLastPoint(int terminalId) async {
    assert(terminalId != null);
    return platform(
      android: (pool) async {
        final completer = Completer<TrackPoint>();
        final request = await com_amap_api_track_query_model_LatestPointRequest
            .create__long__long(_serviceId, terminalId);
        await _androidManager.queryLatestPoint(
          request,
          _androidDelegate
            .._onQueryLastPoint = completer.complete
            .._onRequestFailed = completer.completeError,
        );
        pool.add(request);
        return completer.future;
      },
      ios: (pool) async {
        final completer = Completer<TrackPoint>();

        final request = await AMapTrackQueryLastPointRequest.create__();
        await request.set_serviceID(_serviceId.toString());
        await request.set_terminalID(terminalId.toString());

        await _iosManager.set_delegate(
          _iosDelegate
            .._onQueryLastPoint = completer.complete
            .._onRequestFailed = completer.completeError,
        );

        await _iosManager.AMapTrackQueryLastPoint(request);

        pool.add(request);
        return completer.future;
      },
    );
  }
}

mixin _Pro on _Holder {}
